function API() {

    let apiUrl = 'http://www.localhost:8000'
    let ResourceTranslator = function() {
        var split = this.split('');
        split[0] = split[0].toLowerCase();
        split = split.join("");
        return split.replace(/([A-Z])/g, function($1) {
            return "-" + $1.toLowerCase();
        });
    };

    return {


        setAPIURL: function(val) {
            apiUrl = val;
        },
        setResourceTranslator: function(fn) {
            ResourceTranslator = fn;
        },
        $get: ['$q', '$http', function($q, $http) {

            String.prototype.toDash = ResourceTranslator;

            return class API {
                constructor(Resource) {
                    this.Resource = Resource;
                    this.apiUrl = apiUrl;
                    this.resourceUrl = this.constructor._URLBuilder(Resource);
                }

                static get URL(){
                    return apiUrl
                }
                static _URLBuilder(Resource, pathData) {
                    var resource = Resource.name.split('_')[0].toDash() + 's/';
                    var args;
                    var URL = apiUrl + resource;
                    if (pathData && pathData.length > 0) { //additional url parameters are given. If resource, dashify, if not, treat as action/param.
                        for (var x = 0; x < pathData.length; x++) {
                            if (typeof(pathData[x]) == 'string' || typeof(pathData[x]) == 'number') {
                                URL = URL + pathData[x] + '/'
                            } else {
                                if (pathData[x] instanceof Function) {
                                    URL = URL + pathData[x].name.split('_')[0].toDash() + 's/'
                                } else {
                                    URL = URL + pathData[x].constructor.name.split('_')[0].toDash() + 's/';
                                    if (pathData[x].id) URL = URL + pathData[x].id + '/';

                                }
                            }
                        }
                    }
                    return URL
                }

                static _errorHandler(resourceName, $q, response) {
                    let re = /(API.\w+)/g;
                    let aRegexResult = re.exec(this.stack);
                    let methodName = aRegexResult[0] || aRegexResult[1];
                    console.error('APIService failure at method: "' + methodName + '" for resource: "' + resourceName + '\".' + '\n' + 'status: ' + response.status + ' statusText: ' + response.statusText)
                    if (typeof(response.data) == 'string') {
                        console.error(response.data)
                    } else {
                        for (var field in response.data) {
                            console.error('Error message -- ' + field + ': ' + response.data[field].toString())
                        }
                    }
                    // throw Error('test')
                    return $q.reject(response);
                }

                static _successHandler(mode, response) {
                    if (mode == 'delete') return $q.when(response)
                    return $q.when(response.data);
                }

                get(instance, args) {
                    var params = { params: args.params };
                    var pathData;
                    if (instance) { // if no instance, get hit index route instead of details route.
                        if (args.nestedResource) pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
                        if (args.recursive) Object.assign(instance, args.recursive) // @@TODO@@ nestedResource should be empty if recursive is given. Check this
                        if (!instance.id) delete(instance.id)
                    }
                    var reqObj = {
                        method: 'GET',
                        url: this.constructor._URLBuilder(this.Resource, pathData),
                    }
                    Object.assign(reqObj, params);
                    return $http(reqObj).then(this.constructor._successHandler.bind(this, 'get'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))
                }

                create(instance, args) {
                    const params = { params: args.params };
                    var pathData;
                    if (args.nestedResource) {
                        pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
                        if (args.nestedResource.id == undefined) {
                            // to cover the case where .addRelation is used on an unsaved object;
                            instance = args.nestedResource
                        }
                    }
                    if (args.recursive) Object.assign(instance, args.recursive)
                    if (!instance.id) delete(instance.id)
                    const reqObj = {
                        method: 'POST',
                        url: this.constructor._URLBuilder(this.Resource, pathData),
                        data: instance
                    }
                    Object.assign(reqObj, params);
                    return $http(reqObj)
                        .then(this.constructor._successHandler.bind(this, 'create'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))
                }
                delete(instance, args) {
                    const params = { params: args.params };
                    var pathData;
                    if (args.nestedResource) {
                        pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
                    } else {
                        pathData = [instance.id];
                    }
                    const reqObj = {
                        method: 'DELETE',
                        url: this.constructor._URLBuilder(this.Resource, pathData)
                    }
                    Object.assign(reqObj, params);
                    return $http(reqObj)
                        .then(this.constructor._successHandler.bind(this, 'delete'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))

                }
                put(instance, args) {
                    const params = { params: args.params };
                    var pathData;
                    if (args.nestedResource) {
                        pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
                    } else {
                        pathData = [instance.id];
                    }
                    instance = args.nestedResource || instance
                    const reqObj = {
                        method: 'PUT',
                        url: this.constructor._URLBuilder(this.Resource, pathData),
                        data: instance
                    }
                    Object.assign(reqObj, params);
                    return $http(reqObj)
                        .then(this.constructor._successHandler.bind(this, 'put'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))
                }
            }


        }]


    }

}


export default API


// function API() {

//     let apiUrl = 'http://www.localhost:8000'
//     let ResourceTranslator = function() {
//         var split = this.split('');
//         split[0] = split[0].toLowerCase();
//         split = split.join("");
//         return split.replace(/([A-Z])/g, function($1) {
//             return "-" + $1.toLowerCase();
//         });
//     };

//     return {


//         setAPIURL: function(val) {
//             apiUrl = val;
//         },
//         setResourceTranslator: function(fn) {
//             ResourceTranslator = fn;
//         },
//         $get: ['$q', '$http', function($q, $http) {

//             String.prototype.toDash = ResourceTranslator;

//             return class API {
//                 constructor(Resource) {
//                     this.Resource = Resource;
//                 }

//                 static get URL(){
//                     return apiUrl
//                 }
//                 static _URLBuilder(Resource, pathData) {
//                     var resource = Resource.name.split('_')[0].toDash() + 's/';
//                     var args;
//                     var URL = apiUrl + resource;
//                     if (pathData && pathData.length > 0) { //additional url parameters are given. If resource, dashify, if not, treat as action/param.
//                         for (var x = 0; x < pathData.length; x++) {
//                             if (typeof(pathData[x]) == 'string' || typeof(pathData[x]) == 'number') {
//                                 URL = URL + pathData[x] + '/'
//                             } else {
//                                 if (pathData[x] instanceof Function) {
//                                     URL = URL + pathData[x].name.split('_')[0].toDash() + 's/'
//                                 } else {
//                                     URL = URL + pathData[x].constructor.name.split('_')[0].toDash() + 's/';
//                                     if (pathData[x].id) URL = URL + pathData[x].id + '/';

//                                 }
//                             }
//                         }
//                     }
//                     return URL
//                 }

//                 static _errorHandler(resourceName, $q, response) {
//                     let re = /(API.\w+)/g;
//                     let aRegexResult = re.exec(this.stack);
//                     let methodName = aRegexResult[0] || aRegexResult[1];
//                     console.error('APIService failure at method: "' + methodName + '" for resource: "' + resourceName + '\".' + '\n' + 'status: ' + response.status + ' statusText: ' + response.statusText)
//                     if (typeof(response.data) == 'string') {
//                         console.error(response.data)
//                     } else {
//                         for (var field in response.data) {
//                             console.error('Error message -- ' + field + ': ' + response.data[field].toString())
//                         }
//                     }
//                     return $q.reject(response);
//                 }

//                 static _successHandler(mode, response) {
//                     if (mode == 'delete') return $q.when(response)
//                     return $q.when(response.data);
//                 }

//                 get(instance, args) {
//                     var params = { params: args.params };
//                     var pathData;
//                     if (instance) { // if no instance, get hit index route instead of details route.
//                         if (args.nestedResource) pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
//                         if (args.recursive) Object.assign(instance, args.recursive) // @@TODO@@ nestedResource should be empty if recursive is given. Check this
//                         if (!instance.id) delete(instance.id)
//                     }
//                     var reqObj = {
//                         method: 'GET',
//                         url: this.constructor._URLBuilder(this.Resource, pathData),
//                     }
//                     Object.assign(reqObj, params);
//                     return $http(reqObj).then(this.constructor._successHandler.bind(this, 'get'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))
//                 }

//                 create(instance, args) {
//                     const params = { params: args.params };
//                     var pathData;
//                     if (args.nestedResource) {
//                         pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
//                         if (args.nestedResource.id == undefined) {
//                             // to cover the case where .addRelation is used on an unsaved object;
//                             instance = args.nestedResource
//                         }
//                     }
//                     if (args.recursive) Object.assign(instance, args.recursive)
//                     if (!instance.id) delete(instance.id)
//                     const reqObj = {
//                         method: 'POST',
//                         url: this.constructor._URLBuilder(this.Resource, pathData),
//                         data: instance
//                     }
//                     Object.assign(reqObj, params);
//                     return $http(reqObj)
//                         .then(this.constructor._successHandler.bind(this, 'create'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))
//                 }
//                 delete(instance, args) {
//                     const params = { params: args.params };
//                     var pathData;
//                     if (args.nestedResource) {
//                         pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
//                     } else {
//                         pathData = [instance.id];
//                     }
//                     const reqObj = {
//                         method: 'DELETE',
//                         url: this.constructor._URLBuilder(this.Resource, pathData)
//                     }
//                     Object.assign(reqObj, params);
//                     return $http(reqObj)
//                         .then(this.constructor._successHandler.bind(this, 'delete'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))

//                 }
//                 put(instance, args) {
//                     const params = { params: args.params };
//                     var pathData;
//                     if (args.nestedResource) {
//                         pathData = Array.prototype.concat.apply([], [instance.id, args.nestedResource]);
//                     } else {
//                         pathData = [instance.id];
//                     }
//                     instance = args.nestedResource || instance
//                     const reqObj = {
//                         method: 'PUT',
//                         url: this.constructor._URLBuilder(this.Resource, pathData),
//                         data: instance
//                     }
//                     Object.assign(reqObj, params);
//                     return $http(reqObj)
//                         .then(this.constructor._successHandler.bind(this, 'put'), this.constructor._errorHandler.bind(new Error(), this.Resource.name, $q))
//                 }
//             }


//         }]


//     }

// }


// export default API
