function Base($q, API, RelationCollection) {


    return class Base {

        static factory(args) {
            args = Array.prototype.concat.apply([], [args])
            return args.map(function(inst, i) {
                return new this(inst);
            }, this)
        }
        static byId(args) {
            var match = false;
            for (var i = 0; i < this.list.length; i++) {
                if (this.list[i].id == args) {
                    match = this.list[i];
                    break;
                }
            }
            return match;
        }

        static get(args) {
            function defaultCallback(response) {

                var instances = this.factory(response)
                this.alreadyFetched = true;
                return $q.when(instances)
            }
            args = args || {};
            return this.API.get(null, args).then(defaultCallback.bind(this));
        }

        static sort(target, type) {
            this.list.sort(function(x, y) {
                if (target == 'id') {
                    if (type == 'desc') {
                        let c = x;
                        x = y;
                        y = c;
                    }
                    return x.id - y.id
                }
                if (target == 'name') {
                    if (type == 'desc') {
                        let c = x;
                        x = y;
                        y = c;
                    }
                    return x.id - y.id
                }
            })
        }

        static map(attr) {
            var _createVariableMapping = this._createVariableMapping.bind(this, attr);
            return { to: _createVariableMapping };
        }
        static relate(attr) {
            var _createRelation = this._createRelation.bind(this, attr);
            return { to: _createRelation };
        }

        get(args) {
            var instance = this;
            // var requestInstance = this.constructor._preprocessOutgoing(this); no preprocess outgoing needed; only ID.
            function defaultCallback(response) {
                instance.id = response.id;
                return $q.when(instance);
            }
            args = args || {};
            if (args.nestedResource && !args.callBack || args.recursive && !args.callBack) {
                throw new NoCallBackError()
            }
            return this.constructor.API.get(instance, args).then(args.callBack || defaultCallback.bind(this));
        }

        save(args) {
            var instance = this;
            var requestInstance = this.constructor._preprocessOutgoing(this);

            function defaultCallback(response) {
                instance.id = response.id
                this.constructor._add(this)
                return $q.when(instance)
            }
            args = args || {};
            if (args.nestedResource && !args.callBack || args.recursive && !args.callBack) {
                throw new NoCallBackError()
            }
            
            // if (args.recursive) debugger;
            return this.constructor.API.create(requestInstance, args).then(args.callBack || defaultCallback.bind(this));
        }

        delete(args) {
            function defaultCallback(instance) {
                instance = this;
                let j = this.constructor.list.findIndex(iterator => iterator.id == instance.id);
                if (j > -1) {
                    this.constructor.list.splice(j, 1);
                }
                return $q.when(instance);
            }
            args = args || {};
            if (args.nestedResource && !args.callBack) {
                throw new NoCallBackError()
            }
            return this.constructor.API.delete(this, args).then(args.callBack || defaultCallback.bind(this))
        }
        put(args) {
            var instance = this;
            var requestInstance = this.constructor._preprocessOutgoing(this);

            function defaultCallback(response) {
                let instance = new this.constructor(response); // if id is not unique, overwrite list entry
                // @@TODO@@ compare every field;
                return $q.when(instance);
            }
            args = args || {};
            if (args.nestedResource && !args.callBack) {
                throw new NoCallBackError()
            }
            return this.constructor.API.put(requestInstance, args).then(args.callBack || defaultCallback.bind(this))
        }
        static _add(instance) { // internally add to .list list
            if (instance.id) { // only saved objects (i.e. with id), should be added to the list
                var notUnique = false
                for (var n = 0; n < this.list.length; n++) {
                    if (this.list[n].id == instance.id) {
                        notUnique = true;
                        break;
                    }
                }
                if (notUnique) {
                    this.list[n] = instance;
                } else {
                    this.list.unshift(instance)
                }
            }
        }
        static _setStaticVars(Resource) {
            if (!Resource.API) Resource.API = new API(Resource);
            if (!Resource.list) Resource.list = [];
            if (!Resource._relations) Resource._relations = {};
            // if (!Resource._recursiveRelations) Resource._recursiveRelations = {};
            if (!Resource._variableMapping) Resource._variableMapping = { _inverse: {} };
            if (!Resource.dataTemplate) Resource.dataTemplate = { 'id': 'optional' };
        }


        static _createVariableMapping(attr, target) {
            this._variableMapping[attr] = target;
            this._variableMapping._inverse[target] = attr;
        }
        static _createRelation(attr, targetResource, options) {
            this._relations[attr] = { resource: targetResource, recursive: false };
            if (options && options.recursive) {
                if (options.recursive === true) {
                    this._relations[attr].recursive = 'parentId'
                } else {
                    this._relations[attr].recursive = options.recursive
                }
            }
        }
        _createRelationCollectionFor(targetResource, items) {
            let _relationSettings;
            for (let label in this.constructor._relations) {
                if (this.constructor._relations[label].resource === targetResource) {
                    _relationSettings = this.constructor._relations[label];
                    break;
                }
            }
            return new RelationCollection(this, _relationSettings, items)
        }
        static setDataTemplate(keys) {
            for (var i = 0; i < keys.length; i++) {
                this.dataTemplate[keys[i]] = 'required';
            }
        }
        static _preprocessOutgoing(instance) {
            var data = {};
            for (var attr in this.dataTemplate) {
                if (this._variableMapping.hasOwnProperty(attr)) {
                    data[attr] = instance[this._variableMapping[attr]] || null
                } else {
                    data[attr] = instance[attr] || null
                }
            }
            return data;
        }
        static _preprocessIncoming(data) {
            var preprocessedData = [];
            var varMap = this._variableMapping;
            data = Array.prototype.concat.apply([], [data])
            for (var i = 0; i < data.length; i++) {
                var preprocessedRawInstance = {}
                var rawInstance = data[i];
                for (var attr in this.dataTemplate) {
                    if (!rawInstance.hasOwnProperty(attr) && this.dataTemplate[attr] == 'required') {
                        if (rawInstance.hasOwnProperty(this._variableMapping[attr])) {
                            rawInstance[attr] = rawInstance[this._variableMapping[attr]]
                        } else {
                            throw Error('Expecting raw instance input to include key "' + attr + '", but it was not found (' + this.name + ')')
                        }
                    }
                    if (Object.getOwnPropertyNames(varMap).length > 1 && varMap.hasOwnProperty(attr)) {
                        preprocessedRawInstance[varMap[attr]] = rawInstance[attr];
                    } else {
                        preprocessedRawInstance[attr] = rawInstance[attr];
                    }
                    if (this._relations.hasOwnProperty(attr) && !!preprocessedRawInstance[attr]) preprocessedRawInstance[attr] = this._relations[attr].resource._preprocessIncoming(rawInstance[attr]);

                }
                preprocessedData.push(preprocessedRawInstance);
            }
            // @@TODO@@ throw error when there are keys from api response that are not in this.keys
            if (preprocessedData.length == 1) return preprocessedData[0];
            return preprocessedData;
        }
    }


}



export default Base
